@@ -183,13 +183,16 @@ static MagickBooleanType DecodeImage(Image *image,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
-  (void) ResetMagickMemory(pixels,0,number_pixels*sizeof(*pixels));
+  (void) memset(pixels,0,number_pixels*sizeof(*pixels));
   byte=0;
   x=0;
   p=pixels;
   q=pixels+number_pixels;
   for (y=0; y < (ssize_t) image->rows; )
   {
+    MagickBooleanType
+      status;
+
     if ((p < pixels) || (p >= q))
       break;
     count=ReadBlobByte(image);
@@ -197,10 +200,10 @@ static MagickBooleanType DecodeImage(Image *image,
       break;
     if (count > 0)
       {
-        count=(int) MagickMin((size_t) count,(size_t) (q-p));
         /*
           Encoded mode.
         */
+        count=(int) MagickMin((size_t) count,(size_t) (q-p));
         byte=ReadBlobByte(image);
         if (byte == EOF)
           break;
@@ -223,6 +226,8 @@ static MagickBooleanType DecodeImage(Image *image,
           Escape mode.
         */
         count=ReadBlobByte(image);
+        if (count == EOF)
+          break;
         if (count == 0x01)
           return(MagickTrue);
         switch (count)
@@ -291,12 +296,14 @@ static MagickBooleanType DecodeImage(Image *image,
           }
         }
       }
-    if (SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,image->rows) == MagickFalse)
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
-  return(MagickTrue);
+  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
 }
 
 /*
@@ -511,7 +518,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine if this a DIB file.
   */
-  (void) ResetMagickMemory(&dib_info,0,sizeof(dib_info));
+  (void) memset(&dib_info,0,sizeof(dib_info));
   dib_info.size=ReadBlobLSBLong(image);
   if (dib_info.size != 40)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -1175,7 +1182,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image,
     sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(pixels,0,dib_info.image_size);
+  (void) memset(pixels,0,dib_info.image_size);
   switch (dib_info.bits_per_pixel)
   {
     case 1:
