@@ -347,7 +347,7 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,
         }
       else
         for (p=image->directory; *p != '\0'; p++)
-          if (*p != '\377')
+          if (*p != '\xff')
             (void) WriteBlobByte(image,(unsigned char) *p);
           else
             {
@@ -422,7 +422,7 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,
     }
   else
     for (p=image->directory; *p != '\0'; p++)
-      if (*p != '\377')
+      if (*p != '\xff')
         (void) WriteBlobByte(image,(unsigned char) *p);
       else
         {
