@@ -178,7 +178,7 @@ static MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
 
   if (image->alpha_trait == UndefinedPixelTrait)
     ThrowWriterException(CoderError,"ImageDoesNotHaveAnAlphaChannel");
-  matte_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
+  matte_image=CloneImage(image,0,0,MagickTrue,exception);
   if (matte_image == (Image *) NULL)
     return(MagickFalse);
   (void) SetImageType(matte_image,TrueColorAlphaType,exception);
