@@ -213,13 +213,14 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   *id='\0';
-  (void) ResetMagickMemory(keyword,0,sizeof(keyword));
+  (void) memset(keyword,0,sizeof(keyword));
   offset=0;
   do
   {
     /*
       Decode image header;  header terminates one character beyond a ':'.
     */
+    SetGeometryInfo(&geometry_info);
     profiles=(LinkedListInfo *) NULL;
     length=MagickPathExtent;
     options=AcquireString((char *) NULL);
@@ -278,7 +279,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Get the keyword.
             */
-            length=MagickPathExtent;
+            length=MagickPathExtent-1;
             p=keyword;
             do
             {
@@ -571,6 +572,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     image->number_meta_channels=StringToUnsignedLong(options);
                     if (image->number_meta_channels > MaxPixelChannels)
                       {
+                        if (profiles != (LinkedListInfo *) NULL)
+                          profiles=DestroyLinkedList(profiles,
+                            RelinquishMagickMemory);
                         options=DestroyString(options);
                         ThrowReaderException(CorruptImageError,
                           "ImproperImageHeader");
@@ -778,10 +782,13 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Verify that required image information is defined.
     */
-    if ((LocaleCompare(id,"MagickCache") != 0) ||
+    if ((LocaleCompare(id,"MagickCache") != 0) || (image->depth > 128) ||
         (image->storage_class == UndefinedClass) ||
-        (image->compression == UndefinedCompression) || (image->columns == 0) ||
-        (image->rows == 0))
+        (image->compression == UndefinedCompression) ||
+        (image->columns == 0) || (image->rows == 0) ||
+        (image->number_channels > MaxPixelChannels) ||
+        (image->number_meta_channels > (MaxPixelChannels-8)) ||
+        ((image->number_channels+image->number_meta_channels) >= MaxPixelChannels))
       {
         if (profiles != (LinkedListInfo *) NULL)
           profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
@@ -825,10 +832,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               p=image->directory+strlen(image->directory);
             }
           c=ReadBlobByte(image);
+          if (c == EOF)
+            break;
           *p++=(char) c;
         } while (c != (int) '\0');
       }
-   if (profiles != (LinkedListInfo *) NULL)
+    if (profiles != (LinkedListInfo *) NULL)
       {
         const char
           *name;
@@ -1131,7 +1140,8 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image,
     i;
 
   size_t
-    depth;
+    depth,
+    imageListLength;
 
   /*
     Open persistent cache.
@@ -1151,6 +1161,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image,
   AppendImageFormat("cache",cache_filename);
   scene=0;
   offset=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     /*
@@ -1521,7 +1532,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image,
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
+          imageListLength,image->client_data);
         if (status == MagickFalse)
           break;
       }
