@@ -394,9 +394,11 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
 }
 
 static unsigned char *DecodeImage(Image *blob,Image *image,
-  size_t bytes_per_line,const unsigned int bits_per_pixel,size_t *extent,
-  ExceptionInfo *exception)
+  size_t bytes_per_line,const unsigned int bits_per_pixel,size_t *extent)
 {
+  MagickBooleanType
+    status;
+
   MagickSizeType
     number_pixels;
 
@@ -451,7 +453,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   if (pixels == (unsigned char *) NULL)
     return((unsigned char *) NULL);
   *extent=row_bytes*image->rows*sizeof(*pixels);
-  (void) ResetMagickMemory(pixels,0,*extent);
+  (void) memset(pixels,0,*extent);
   scanline=(unsigned char *) AcquireQuantumMemory(row_bytes,2*
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
@@ -459,7 +461,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       return((unsigned char *) NULL);
     }
-  (void) ResetMagickMemory(scanline,0,2*row_bytes*sizeof(*scanline));
+  (void) memset(scanline,0,2*row_bytes*sizeof(*scanline));
+  status=MagickTrue;
   if (bytes_per_line < 8)
     {
       /*
@@ -467,27 +470,25 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       */
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=pixels+y*width*GetPixelChannels(image);;
+        q=pixels+y*width*GetPixelChannels(image);
         number_pixels=bytes_per_line;
         count=ReadBlob(blob,(size_t) number_pixels,scanline);
         if (count != (ssize_t) number_pixels)
           {
-            (void) ThrowMagickException(exception,GetMagickModule(),
-              CorruptImageError,"UnableToUncompressImage","`%s'",
-              image->filename);
+            status=MagickFalse;
             break;
           }
         p=ExpandBuffer(scanline,&number_pixels,bits_per_pixel);
         if ((q+number_pixels) > (pixels+(*extent)))
           {
-            (void) ThrowMagickException(exception,GetMagickModule(),
-              CorruptImageError,"UnableToUncompressImage","`%s'",
-              image->filename);
+            status=MagickFalse;
             break;
           }
-        (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+        (void) memcpy(q,p,(size_t) number_pixels);
       }
       scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      if (status == MagickFalse)
+        pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       return(pixels);
     }
   /*
@@ -499,18 +500,16 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     if (bytes_per_line > 200)
       scanline_length=ReadBlobMSBShort(blob);
     else
-      scanline_length=1UL*ReadBlobByte(blob);
-    if (scanline_length >= row_bytes)
+      scanline_length=(size_t) ReadBlobByte(blob);
+    if ((scanline_length >= row_bytes) || (scanline_length == 0))
       {
-        (void) ThrowMagickException(exception,GetMagickModule(),
-          CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
+        status=MagickFalse;
         break;
       }
     count=ReadBlob(blob,scanline_length,scanline);
     if (count != (ssize_t) scanline_length)
       {
-        (void) ThrowMagickException(exception,GetMagickModule(),
-          CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
+        status=MagickFalse;
         break;
       }
     for (j=0; j < (ssize_t) scanline_length; )
@@ -520,7 +519,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           number_pixels=length*bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
           if ((q-pixels+number_pixels) <= *extent)
-            (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+            (void) memcpy(q,p,(size_t) number_pixels);
           q+=number_pixels;
           j+=(ssize_t) (length*bytes_per_pixel+1);
         }
@@ -532,13 +531,15 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           for (i=0; i < (ssize_t) length; i++)
           {
             if ((q-pixels+number_pixels) <= *extent)
-              (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+              (void) memcpy(q,p,(size_t) number_pixels);
             q+=number_pixels;
           }
           j+=(ssize_t) bytes_per_pixel+1;
         }
   }
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  if (status == MagickFalse)
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   return(pixels);
 }
 
@@ -801,8 +802,9 @@ static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
   rectangle->left=(short) ReadBlobMSBShort(image);
   rectangle->bottom=(short) ReadBlobMSBShort(image);
   rectangle->right=(short) ReadBlobMSBShort(image);
-  if ((EOFBlob(image) != MagickFalse) || (rectangle->left > rectangle->right) ||
-      (rectangle->top > rectangle->bottom))
+  if ((EOFBlob(image) != MagickFalse) || 
+      ((rectangle->bottom-rectangle->top) <= 0) ||
+      ((rectangle->right-rectangle->left) <= 0))
     return(MagickFalse);
   return(MagickTrue);
 }
@@ -810,12 +812,25 @@ static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
 static Image *ReadPICTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+#define ThrowPICTException(exception,message) \
+{ \
+  if (tile_image != (Image *) NULL) \
+    tile_image=DestroyImage(tile_image); \
+  if (read_info != (ImageInfo *) NULL) \
+    read_info=DestroyImageInfo(read_info); \
+  ThrowReaderException((exception),(message)); \
+}
+
   char
     geometry[MagickPathExtent],
     header_ole[4];
 
   Image
-    *image;
+    *image,
+    *tile_image;
+
+  ImageInfo
+    *read_info;
 
   int
     c,
@@ -875,6 +890,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   /*
     Read PICT header.
   */
+  read_info=(ImageInfo *) NULL;
+  tile_image=(Image *) NULL;
   pixmap.bits_per_pixel=0;
   pixmap.component_count=0;
   /*
@@ -891,31 +908,31 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         break;
   (void) ReadBlobMSBShort(image);  /* skip picture size */
   if (ReadRectangle(image,&frame) == MagickFalse)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   while ((c=ReadBlobByte(image)) == 0) ;
   if (c != 0x11)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  version=ReadBlobByte(image);
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+  version=(ssize_t) ReadBlobByte(image);
   if (version == 2)
     {
       c=ReadBlobByte(image);
       if (c != 0xff)
-        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        ThrowPICTException(CorruptImageError,"ImproperImageHeader");
     }
   else
     if (version != 1)
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) ||
       (frame.bottom < 0) || (frame.left >= frame.right) ||
       (frame.top >= frame.bottom))
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   /*
     Create black canvas.
   */
   flags=0;
   image->depth=8;
-  image->columns=1UL*(frame.right-frame.left);
-  image->rows=1UL*(frame.bottom-frame.top);
+  image->columns=(size_t) (frame.right-frame.left);
+  image->rows=(size_t) (frame.bottom-frame.top);
   image->resolution.x=DefaultResolution;
   image->resolution.y=DefaultResolution;
   image->units=UndefinedResolution;
@@ -926,10 +943,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         return(GetFirstImageInList(image));
       }
   status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,exception);
   if (status == MagickFalse)
     return(DestroyImageList(image));
-  image->alpha_trait=BlendPixelTrait;
-  (void) SetImageBackgroundColor(image,exception);
   /*
     Interpret PICT opcodes.
   */
@@ -945,6 +962,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       code=ReadBlobMSBSignedShort(image);
     if (code < 0)
       break;
+    if (code == 0)
+      continue;
     if (code > 0xa1)
       {
         if (image->debug != MagickFalse)
@@ -971,15 +990,16 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (((frame.left & 0x8000) != 0) || ((frame.top & 0x8000) != 0))
               break;
-            image->columns=1UL*(frame.right-frame.left);
-            image->rows=1UL*(frame.bottom-frame.top);
+            image->columns=(size_t) (frame.right-frame.left);
+            image->rows=(size_t) (frame.bottom-frame.top);
             status=SetImageExtent(image,image->columns,image->rows,exception);
+            if (status != MagickFalse)
+              status=ResetImagePixels(image,exception);
             if (status == MagickFalse)
               return(DestroyImageList(image));
-            (void) SetImageBackgroundColor(image,exception);
             break;
           }
           case 0x12:
@@ -996,7 +1016,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             /*
               Skip pattern definition.
             */
-            pattern=1L*ReadBlobMSBShort(image);
+            pattern=(ssize_t) ReadBlobMSBShort(image);
             for (i=0; i < 8; i++)
               if (ReadBlobByte(image) == EOF)
                 break;
@@ -1008,23 +1028,23 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (pattern != 1)
-              ThrowReaderException(CorruptImageError,"UnknownPatternType");
+              ThrowPICTException(CorruptImageError,"UnknownPatternType");
             length=ReadBlobMSBShort(image);
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (ReadPixmap(image,&pixmap) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-            image->depth=1UL*pixmap.component_size;
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+            image->depth=(size_t) pixmap.component_size;
             image->resolution.x=1.0*pixmap.horizontal_resolution;
             image->resolution.y=1.0*pixmap.vertical_resolution;
             image->units=PixelsPerInchResolution;
             (void) ReadBlobMSBLong(image);
-            flags=1L*ReadBlobMSBShort(image);
+            flags=(ssize_t) ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
             for (i=0; i <= (ssize_t) length; i++)
               (void) ReadBlobMSBLong(image);
-            width=1UL*(frame.bottom-frame.top);
-            height=1UL*(frame.right-frame.left);
+            width=(size_t) (frame.bottom-frame.top);
+            height=(size_t) (frame.right-frame.left);
             if (pixmap.bits_per_pixel <= 8)
               length&=0x7fff;
             if (pixmap.bits_per_pixel == 16)
@@ -1038,7 +1058,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     break;
               }
             else
-              for (i=0; i < (int) height; i++)
+              for (i=0; i < (ssize_t) height; i++)
+              {
+                if (EOFBlob(image) != MagickFalse)
+                  break;
                 if (length > 200)
                   {
                     for (j=0; j < (ssize_t) ReadBlobMSBShort(image); j++)
@@ -1049,6 +1072,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   for (j=0; j < (ssize_t) ReadBlobByte(image); j++)
                     if (ReadBlobByte(image) == EOF)
                       break;
+              }
             break;
           }
           case 0x1b:
@@ -1089,9 +1113,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           case 0x9a:
           case 0x9b:
           {
-            Image
-              *tile_image;
-
             PICTRectangle
               source,
               destination;
@@ -1113,7 +1134,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             */
             bytes_per_line=0;
             if ((code != 0x9a) && (code != 0x9b))
-              bytes_per_line=1L*ReadBlobMSBShort(image);
+              bytes_per_line=(ssize_t) ReadBlobMSBShort(image);
             else
               {
                 (void) ReadBlobMSBShort(image);
@@ -1121,33 +1142,28 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 (void) ReadBlobMSBShort(image);
               }
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             /*
               Initialize tile image.
             */
-            tile_image=CloneImage(image,1UL*(frame.right-frame.left),
-              1UL*(frame.bottom-frame.top),MagickTrue,exception);
+            tile_image=CloneImage(image,(size_t) (frame.right-frame.left),
+              (size_t) (frame.bottom-frame.top),MagickTrue,exception);
             if (tile_image == (Image *) NULL)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if ((code == 0x9a) || (code == 0x9b) ||
                 ((bytes_per_line & 0x8000) != 0))
               {
                 if (ReadPixmap(image,&pixmap) == MagickFalse)
-                  {
-                    tile_image=DestroyImage(tile_image);
-                    ThrowReaderException(CorruptImageError,
-                      "ImproperImageHeader");
-                  }
-                tile_image->depth=1UL*pixmap.component_size;
+                  ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+                tile_image->depth=(size_t) pixmap.component_size;
                 tile_image->alpha_trait=pixmap.component_count == 4 ?
                   BlendPixelTrait : UndefinedPixelTrait;
                 tile_image->resolution.x=(double) pixmap.horizontal_resolution;
                 tile_image->resolution.y=(double) pixmap.vertical_resolution;
                 tile_image->units=PixelsPerInchResolution;
                 if (tile_image->alpha_trait != UndefinedPixelTrait)
-                  image->alpha_trait=tile_image->alpha_trait;
+                  (void) SetImageAlpha(tile_image,OpaqueAlpha,exception);
               }
-            (void) SetImageBackgroundColor(tile_image,exception);
             if ((code != 0x9a) && (code != 0x9b))
               {
                 /*
@@ -1157,17 +1173,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if ((bytes_per_line & 0x8000) != 0)
                   {
                     (void) ReadBlobMSBLong(image);
-                    flags=1L*ReadBlobMSBShort(image);
+                    flags=(ssize_t) ReadBlobMSBShort(image);
                     tile_image->colors=1UL*ReadBlobMSBShort(image)+1;
                   }
                 status=AcquireImageColormap(tile_image,tile_image->colors,
                   exception);
                 if (status == MagickFalse)
-                  {
-                    tile_image=DestroyImage(tile_image);
-                    ThrowReaderException(ResourceLimitError,
-                      "MemoryAllocationFailed");
-                  }
+                  ThrowPICTException(ResourceLimitError,
+                    "MemoryAllocationFailed");
                 if ((bytes_per_line & 0x8000) != 0)
                   {
                     for (i=0; i < (ssize_t) tile_image->colors; i++)
@@ -1196,16 +1209,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     }
                   }
               }
+            if (EOFBlob(image) != MagickFalse)
+              ThrowPICTException(CorruptImageError,
+                "InsufficientImageDataInFile");
             if (ReadRectangle(image,&source) == MagickFalse)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-              }
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (ReadRectangle(image,&destination) == MagickFalse)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-              }
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             (void) ReadBlobMSBShort(image);
             if ((code == 0x91) || (code == 0x99) || (code == 0x9b))
               {
@@ -1219,17 +1229,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               }
             if ((code != 0x9a) && (code != 0x9b) &&
                 (bytes_per_line & 0x8000) == 0)
-              pixels=DecodeImage(image,tile_image,1UL*bytes_per_line,1,&extent,
-                exception);
+              pixels=DecodeImage(image,tile_image,(size_t) bytes_per_line,1,
+                &extent);
             else
-              pixels=DecodeImage(image,tile_image,1UL*bytes_per_line,1U*
-                pixmap.bits_per_pixel,&extent,exception);
+              pixels=DecodeImage(image,tile_image,(size_t) bytes_per_line,
+                (unsigned int) pixmap.bits_per_pixel,&extent);
             if (pixels == (unsigned char *) NULL)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              }
+              ThrowPICTException(CorruptImageError,"UnableToUncompressImage");
             /*
               Convert PICT tile image to pixel packets.
             */
@@ -1238,8 +1244,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             {
               if (p > (pixels+extent+image->columns))
                 {
-                  tile_image=DestroyImage(tile_image);
-                  ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
+                  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+                  ThrowPICTException(CorruptImageError,"NotEnoughPixelData");
                 }
               q=QueueAuthenticPixels(tile_image,0,y,tile_image->columns,1,
                 exception);
@@ -1249,7 +1255,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 if (tile_image->storage_class == PseudoClass)
                   {
-                    index=ConstrainColormapIndex(tile_image,*p,exception);
+                    index=(Quantum) ConstrainColormapIndex(tile_image,(ssize_t)
+                      *p,exception);
                     SetPixelIndex(tile_image,index,q);
                     SetPixelRed(tile_image,
                       tile_image->colormap[(ssize_t) index].red,q);
@@ -1262,8 +1269,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     if (pixmap.bits_per_pixel == 16)
                       {
-                        i=(*p++);
-                        j=(*p);
+                        i=(ssize_t) (*p++);
+                        j=(size_t) (*p);
                         SetPixelRed(tile_image,ScaleCharToQuantum(
                           (unsigned char) ((i & 0x7c) << 1)),q);
                         SetPixelGreen(tile_image,ScaleCharToQuantum(
@@ -1276,11 +1283,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       if (tile_image->alpha_trait == UndefinedPixelTrait)
                         {
                           if (p > (pixels+extent+2*image->columns))
-                            {
-                              tile_image=DestroyImage(tile_image);
-                              ThrowReaderException(CorruptImageError,
-                                "NotEnoughPixelData");
-                            }
+                            ThrowPICTException(CorruptImageError,
+                              "NotEnoughPixelData");
                           SetPixelRed(tile_image,ScaleCharToQuantum(*p),q);
                           SetPixelGreen(tile_image,ScaleCharToQuantum(
                             *(p+tile_image->columns)),q);
@@ -1290,11 +1294,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       else
                         {
                           if (p > (pixels+extent+3*image->columns))
-                            {
-                              tile_image=DestroyImage(tile_image);
-                              ThrowReaderException(CorruptImageError,
-                                "NotEnoughPixelData");
-                            }
+                            ThrowPICTException(CorruptImageError,
+                              "NotEnoughPixelData");
                           SetPixelAlpha(tile_image,ScaleCharToQuantum(*p),q);
                           SetPixelRed(tile_image,ScaleCharToQuantum(
                             *(p+tile_image->columns)),q);
@@ -1316,16 +1317,18 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   if (p < pixels)
                     break;
                 }
-              status=SetImageProgress(image,LoadImageTag,y,tile_image->rows);
+              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                tile_image->rows);
               if (status == MagickFalse)
                 break;
             }
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-            if (jpeg == MagickFalse)
+            if ((jpeg == MagickFalse) && (EOFBlob(image) == MagickFalse))
               if ((code == 0x9a) || (code == 0x9b) ||
                   ((bytes_per_line & 0x8000) != 0))
                 (void) CompositeImage(image,tile_image,CopyCompositeOp,
-                  MagickTrue,destination.left,destination.top,exception);
+                  MagickTrue,(ssize_t) destination.left,(ssize_t)
+                  destination.top,exception);
             tile_image=DestroyImage(tile_image);
             break;
           }
@@ -1355,8 +1358,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (count != (ssize_t) length)
               {
                 info=(unsigned char *) RelinquishMagickMemory(info);
-                ThrowReaderException(ResourceLimitError,
-                  "UnableToReadImageData");
+                ThrowPICTException(ResourceLimitError,"UnableToReadImageData");
               }
             switch (type)
             {
@@ -1369,7 +1371,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (status == MagickFalse)
                   {
                     info=(unsigned char *) RelinquishMagickMemory(info);
-                    ThrowReaderException(ResourceLimitError,
+                    ThrowPICTException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
                 break;
@@ -1382,7 +1384,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (status == MagickFalse)
                   {
                     info=(unsigned char *) RelinquishMagickMemory(info);
-                    ThrowReaderException(ResourceLimitError,
+                    ThrowPICTException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
                 profile=DestroyStringInfo(profile);
@@ -1423,15 +1425,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       continue;
     if (code == 0x8200)
       {
+        char
+          filename[MaxTextExtent];
+
         FILE
           *file;
 
-        Image
-          *tile_image;
-
-        ImageInfo
-          *read_info;
-
         int
           unique_file;
 
@@ -1442,7 +1441,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         read_info=CloneImageInfo(image_info);
         SetImageInfoBlob(read_info,(void *) NULL,0);
         file=(FILE *) NULL;
-        unique_file=AcquireUniqueFileResource(read_info->filename);
+        unique_file=AcquireUniqueFileResource(filename);
+        (void) FormatLocaleString(read_info->filename,MaxTextExtent,"jpeg:%s",
+          filename);
         if (unique_file != -1)
           file=fdopen(unique_file,"wb");
         if ((unique_file == -1) || (file == (FILE *) NULL))
@@ -1450,10 +1451,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             (void) RelinquishUniqueFileResource(read_info->filename);
             (void) CopyMagickString(image->filename,read_info->filename,
               MagickPathExtent);
-            ThrowFileException(exception,FileOpenError,
-              "UnableToCreateTemporaryFile",image->filename);
-            image=DestroyImageList(image);
-            return((Image *) NULL);
+            ThrowPICTException(FileOpenError,"UnableToCreateTemporaryFile");
           }
         length=ReadBlobMSBLong(image);
         if (length > 154)
@@ -1464,7 +1462,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 (void) fclose(file);
                 (void) RelinquishUniqueFileResource(read_info->filename);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+                ThrowPICTException(CorruptImageError,"ImproperImageHeader");
               }
             for (i=0; i < 122; i++)
               if (ReadBlobByte(image) == EOF)
@@ -1480,7 +1478,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         (void) fclose(file);
         (void) close(unique_file);
         tile_image=ReadImage(read_info,exception);
-        (void) RelinquishUniqueFileResource(read_info->filename);
+        (void) RelinquishUniqueFileResource(filename);
         read_info=DestroyImageInfo(read_info);
         if (tile_image == (Image *) NULL)
           continue;
@@ -1492,7 +1490,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           MagickMax(image->rows,tile_image->rows),exception);
         (void) TransformImageColorspace(image,tile_image->colorspace,exception);
         (void) CompositeImage(image,tile_image,CopyCompositeOp,MagickTrue,
-          frame.left,frame.right,exception);
+          (ssize_t) frame.left,(ssize_t) frame.right,exception);
         image->compression=tile_image->compression;
         tile_image=DestroyImage(tile_image);
         continue;
@@ -1772,12 +1770,12 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  (void) ResetMagickMemory(scanline,0,row_bytes);
-  (void) ResetMagickMemory(packed_scanline,0,(size_t) (row_bytes+MaxCount));
+  (void) memset(scanline,0,row_bytes);
+  (void) memset(packed_scanline,0,(size_t) (row_bytes+MaxCount));
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
-  (void) ResetMagickMemory(buffer,0,PictInfoSize);
+  (void) memset(buffer,0,PictInfoSize);
   (void) WriteBlob(image,PictInfoSize,buffer);
   (void) WriteBlobMSBShort(image,0);
   (void) WriteBlobMSBShort(image,(unsigned short) size_rectangle.top);
@@ -1787,7 +1785,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   (void) WriteBlobMSBShort(image,PictVersion);
   (void) WriteBlobMSBShort(image,0x02ff);  /* version #2 */
   (void) WriteBlobMSBShort(image,PictInfoOp);
-  (void) WriteBlobMSBLong(image,0xFFFE0000UL);
+  (void) WriteBlobMSBLong(image,0xFFFE0000U);
   /*
     Write full size of the file, resolution, frame bounding box, and reserved.
   */
@@ -1818,13 +1816,13 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,(unsigned short)
         (GetStringInfoLength(profile)+4));
-      (void) WriteBlobMSBLong(image,0x00000000UL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
       (void) WriteBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,4);
-      (void) WriteBlobMSBLong(image,0x00000002UL);
+      (void) WriteBlobMSBLong(image,0x00000002U);
     }
   /*
     Write crop region opcode and crop bounding box.
@@ -1867,50 +1865,51 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,PictJPEGOp);
       (void) WriteBlobMSBLong(image,(unsigned int) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,0x00010000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00010000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x40000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00400000UL);
+      (void) WriteBlobMSBLong(image,0x00010000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00010000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x40000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00400000U);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00566A70UL);
-      (void) WriteBlobMSBLong(image,0x65670000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000001UL);
-      (void) WriteBlobMSBLong(image,0x00016170UL);
-      (void) WriteBlobMSBLong(image,0x706C0000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00566A70U);
+      (void) WriteBlobMSBLong(image,0x65670000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000001U);
+      (void) WriteBlobMSBLong(image,0x00016170U);
+      (void) WriteBlobMSBLong(image,0x706C0000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) x_resolution);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) y_resolution);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x87AC0001UL);
-      (void) WriteBlobMSBLong(image,0x0B466F74UL);
-      (void) WriteBlobMSBLong(image,0x6F202D20UL);
-      (void) WriteBlobMSBLong(image,0x4A504547UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x0018FFFFUL);
+      (void) WriteBlobMSBShort(image,0x0000);
+      (void) WriteBlobMSBLong(image,length);
+      (void) WriteBlobMSBShort(image,0x0001);
+      (void) WriteBlobMSBLong(image,0x0B466F74U);
+      (void) WriteBlobMSBLong(image,0x6F202D20U);
+      (void) WriteBlobMSBLong(image,0x4A504547U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x0018FFFFU);
       (void) WriteBlob(image,length,blob);
       if ((length & 0x01) != 0)
         (void) WriteBlobByte(image,'\0');
@@ -1924,7 +1923,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     {
       (void) WriteBlobMSBShort(image,PictPixmapOp);
-      (void) WriteBlobMSBLong(image,(size_t) base_address);
+      (void) WriteBlobMSBLong(image,(unsigned int) base_address);
     }
   (void) WriteBlobMSBShort(image,(unsigned short) (row_bytes | 0x8000));
   (void) WriteBlobMSBShort(image,(unsigned short) bounds.top);
@@ -2010,7 +2009,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     if (image_info->compression == JPEGCompression)
       {
-        (void) ResetMagickMemory(scanline,0,row_bytes);
+        (void) memset(scanline,0,row_bytes);
         for (y=0; y < (ssize_t) image->rows; y++)
           count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
             packed_scanline);
